package Onis::Plugins::Bignumbers;

use strict;
use warnings;

use Onis::Config qw/get_config/;
use Onis::Html qw/html_escape get_filehandle/;
use Onis::Language qw/translate/;
use Onis::Data::Core qw#get_main_nick register_plugin ident_to_name#;

our $DATA;

if (!defined ($DATA->{'nickhash'}))
{
	$DATA->{'count'} = {};
	$DATA->{'nickhash'} = {};
	$DATA->{'nickarray'} = [];
}

$DATA = register_plugin ('TEXT', \&add);
$DATA = register_plugin ('ACTION', \&add);
$DATA = register_plugin ('OUTPUT', \&output);

my $VERSION = '$Id: Bignumbers.pm,v 1.7 2004/04/30 06:56:13 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub add
{
	my $data = shift;

	my $ident = $data->{'ident'};
	my $text = $data->{'text'};

	if ($text =~ m/\b\?/)
	{
		$DATA->{'byident'}{$ident}{'questions'}++;
	}

	if ((uc ($text) eq $text) and ($text =~ m/[A-Z]/))
	{
		$DATA->{'byident'}{$ident}{'uppercase'}++;
	}

	if ($text =~ m/( |^)[;:]-?\)( |$)/)
	{
		$DATA->{'byident'}{$ident}{'smiley_happy'}++;
	}

	if ($text =~ m/( |^):-?\(( |$)/)
	{
		$DATA->{'byident'}{$ident}{'smiley_sad'}++;
	}

	return (1);
}

sub output
{
	my $first;
	my $second;
	my $trans;

	my $fh = get_filehandle ();
	
	$trans = translate ('Big Numbers');
	print $fh <<EOF;
<table class="plugin">
  <tr>
    <th>$trans</th>
  </tr>
EOF
	($first, $second) = sort_by_field ('questions');
	if ($first)
	{
		my $percent = 100 * $DATA->{'byname'}{$first}{'questions'} / $DATA->{'byname'}{$first}{'lines'};
		my $trans = translate ('questions0: %s %2.1f%%');

		print $fh "  <tr>\n    <td>";
		printf $fh ($trans, $first, $percent);
		
		if ($second)
		{
			$percent = 100 * $DATA->{'byname'}{$second}{'questions'} / $DATA->{'byname'}{$second}{'lines'};
			$trans = translate ('questions1: %s %2.1f%%');

			print $fh "<br />\n",
			qq#      <span class="small">#;
			printf $fh ($trans, $second, $percent);
			print $fh '</span>';
		}
		
		print $fh "</td>\n  </tr>\n";
	}

	($first, $second) = sort_by_field ('uppercase');
	if ($first)
	{
		my $percent = 100 * $DATA->{'byname'}{$first}{'uppercase'} / $DATA->{'byname'}{$first}{'lines'};
		my $trans = translate ('yells0: %s %2.1f%%');

		print $fh "  <tr>\n    <td>";
		printf $fh ($trans, $first, $percent);

		if ($second)
		{
			$percent = 100 * $DATA->{'byname'}{$second}{'uppercase'} / $DATA->{'byname'}{$second}{'lines'};
			$trans = translate ('yells1: %s %2.1f%%');

			print $fh "<br />\n",
			qq#      <span class="small">#;
			printf $fh ($trans, $second, $percent);
			print $fh "</span>";
		}

		print $fh "</td>\n  </tr>\n";
	}

	($first, $second) = sort_by_field ('smiley_happy');
	if ($first)
	{
		my $percent = 100 * $DATA->{'byname'}{$first}{'smiley_happy'} / $DATA->{'byname'}{$first}{'lines'};
		my $trans = translate ('happy0: %s %2.1f%%');

		print $fh "  <tr>\n    <td>";
		printf $fh ($trans, $first, $percent);
		
		if ($second)
		{
			$percent = 100 * $DATA->{'byname'}{$second}{'smiley_happy'} / $DATA->{'byname'}{$second}{'lines'};
			$trans = translate ('happy1: %s %2.1f%%');

			print $fh "<br />\n",
			qq#      <span class="small">#;
			printf $fh ($trans, $second, $percent);
			print $fh "</span>";
		}
		
		print $fh "</td>\n  </tr>\n";
	}

	($first, $second) = sort_by_field ('smiley_sad');
	if ($first)
	{
		my $percent = 100 * $DATA->{'byname'}{$first}{'smiley_sad'} / $DATA->{'byname'}{$first}{'lines'};
		my $trans = translate ('sad0: %s %2.1f%%');

		print $fh "  <tr>\n    <td>";
		printf $fh ($trans, $first, $percent);
		
		if ($second)
		{
			$percent = 100 * $DATA->{'byname'}{$second}{'smiley_sad'} / $DATA->{'byname'}{$second}{'lines'};
			$trans = translate ('sad1: %s %2.1f%%');

			print $fh "<br />\n",
			qq#      <span class="small">#;
			printf $fh ($trans, $second, $percent);
			print $fh "</span>";
		}
		
		print $fh "</td>\n  </tr>\n";
	}

	{
		my @names = sort_by_field ('chars');
		
		my $longest = '';
		my $longest2 = '';
		my $shortest = '';
		my $shortest2 = '';
		
		my $chan_chars = 0;
		my $chan_lines = 0;
		
		for (@names)
		{
			$chan_chars += $DATA->{'byname'}{$_}{'chars'};
			$chan_lines += $DATA->{'byname'}{$_}{'lines'};
		}

		if (@names)
		{
			$longest = shift (@names);
		}
		if (@names)
		{
			$longest2 = shift (@names);
		}
		if (@names)
		{
			$shortest = pop (@names);
		}
		if (@names)
		{
			$shortest2 = pop (@names);
		}
		
		if ($longest)
		{
			my $avg = $DATA->{'byname'}{$longest}{'chars'} / $DATA->{'byname'}{$longest}{'lines'};
			my $trans = translate ('max chars0: %s %1.1f');
			
			print $fh "  <tr>\n    <td>";
			printf $fh ($trans, $longest, $avg);
			
			if ($longest2)
			{
				$avg = $DATA->{'byname'}{$longest2}{'chars'} / $DATA->{'byname'}{$longest2}{'lines'};
				$trans = translate ('max chars1: %s %1.1f');

				print $fh "<br />\n",
				qq#      <span class="small">#;
				printf $fh ($trans, $longest2, $avg);
				print $fh "</span>";
			}

			$avg = $chan_chars / $chan_lines;
			$trans = translate ('chars avg: %1.1f');

			print $fh "<br />\n",
			qq#      <span class="small">#;
			printf $fh ($trans, $avg);
			print $fh "</span></td>\n  </tr>\n";
		}

		if ($shortest)
		{
			my $avg = $DATA->{'byname'}{$shortest}{'chars'} / $DATA->{'byname'}{$shortest}{'lines'};
			my $trans = translate ('min chars0: %s %1.1f');
			
			print $fh "  <tr>\n    <td>";
			printf $fh ($trans, $shortest, $avg);
			
			if ($shortest2)
			{
				$avg = $DATA->{'byname'}{$shortest2}{'chars'} / $DATA->{'byname'}{$shortest2}{'lines'};
				$trans = translate ('min chars1: %s %1.1f');

				print $fh "<br />\n",
				qq#      <span class="small">#;
				printf $fh ($trans, $shortest2, $avg);
				print $fh "</span>";
			}
			print $fh "</td>\n  </tr>\n";
		}
	}
	
	{
		my @names = sort_by_field ('words');

		$first = '';
		$second = '';

		my $chan_words = 0;
		my $chan_lines = 0;
		
		for (@names)
		{
			$chan_words += $DATA->{'byname'}{$_}{'words'};
			$chan_lines += $DATA->{'byname'}{$_}{'lines'};
		}
		
		if (@names)
		{
			$first = shift (@names);
		}
		if (@names)
		{
			$second = shift (@names);
		}

		if ($first)
		{
			my $avg = $DATA->{'byname'}{$first}{'words'} / $DATA->{'byname'}{$first}{'lines'};
			my $trans = translate ('max words0: %s %1.1f');
			
			print $fh "  <tr>\n    <td>";
			printf $fh ($trans, $first, $avg);

			if ($second)
			{
				$avg = $DATA->{'byname'}{$second}{'words'} / $DATA->{'byname'}{$second}{'lines'};
				$trans = translate ('max words1: %s %1.1f');

				print $fh "<br />\n",
				qq#      <span class="small">#;
				printf $fh ($trans, $second, $avg);
				print $fh "</span>";
			}

			$avg = $chan_words / $chan_lines;
			$trans = translate ('words avg: %1.1f');
			
			print $fh "<br />\n",
			qq#      <span class="small">#;
			printf $fh ($trans, $avg);
			print $fh "</span></td>\n  </tr>\n";
		}
	}

	print $fh "</table>\n\n";
}

sub sort_by_field
{
	my $field = shift;

	my @retval = sort
	{
		($DATA->{'byname'}{$b}{$field} / $DATA->{'byname'}{$b}{'lines'})
		<=>
		($DATA->{'byname'}{$a}{$field} / $DATA->{'byname'}{$a}{'lines'})
	} grep
	{
		defined ($DATA->{'byname'}{$_}{'lines'})
			and ($DATA->{'byname'}{$_}{'lines'} > 100)
			and defined ($DATA->{'byname'}{$_}{$field})
	}
	(keys (%{$DATA->{'byname'}}));
	
	while (scalar (@retval) < 2)
	{
		push (@retval, '');
	}

	return (@retval);
}
