package Onis::Plugins::Nicks;

use strict;
use warnings;

use Onis::Config qw/get_config/;
use Onis::Html qw/html_escape get_filehandle/;
use Onis::Language qw/translate/;
use Onis::Data::Core qw#register_plugin#;
use Onis::Data::Persistent qw#init#;

our $DATA = init ('$DATA', 'hash');

register_plugin ('TEXT', \&add);
register_plugin ('ACTION', \&add);
register_plugin ('OUTPUT', \&output);

our $TYPE = 'FAST';
if (get_config ('nick_match_type'))
{
	my $tmp = get_config ('nick_match_type');
	$tmp = uc ($tmp);

	if ($tmp eq 'FAST' or $tmp eq 'GOOD')
	{
		$TYPE = $tmp;
	}
	else
	{
		print STDERR $/, __FILE__, ": ``$tmp'' is not a valid value for ``nick_match_type''.",
		$/, __FILE__, ": Use either ``FAST'' (default) or ``GOOD'' (slow).";
	}
}

my $VERSION = '$Id: Nicks.pm,v 1.5 2004/01/18 19:56:12 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub add
{
	my $data = shift;

	my $nick = $data->{'nick'};
	my $hour = $data->{'hour'};
	my $minute = $data->{'minute'};
	my $host = $data->{'host'};
	my $text = $data->{'text'};

	if (!defined ($DATA->{'nickcount'}{$nick}))
	{
		$DATA->{'nickcount'}{$nick} = 0;
	}
	$DATA->{'nickcount'}{$nick}++;
	
	if ($TYPE eq 'GOOD')
	{
		if (!defined ($DATA->{'nickre'}{$nick}))
		{
			my $tmp = '\b' . quotemeta ($nick) . '\b';
			$DATA->{'nickre'}{$nick} = qr#$tmp#;

			push (@{$DATA->{'nickarray'}}, $nick);
		}

		for (@{$DATA->{'nickarray'}})
		{
			my $match_nick = $_;
			next if ($match_nick eq $nick);
	
			my $re = $DATA->{'nickre'}{$match_nick};
	
			if ($text =~ m#$re#)
			{
				if (!defined ($DATA->{'count'}{$match_nick}))
				{
					$DATA->{'count'}{$match_nick} =
					{
						usedby	=> $nick,
						count	=> 1
					}
				}
				else
				{
					$DATA->{'count'}{$match_nick}{'usedby'} = $nick;
					$DATA->{'count'}{$match_nick}{'count'}++;
				}
			}
		}
	}
	elsif ($TYPE eq 'FAST')
	{
		my @potential_nicks = split (/[^\w\`\~\^\-\|\[\]]+/, $text);
		#my @potential_nicks = split (/[\s!\?\.":]+/, $text);

		for (@potential_nicks)
		{
			#s/[!\?\."]+//g;
			my $pot_nick = $_;

			if (defined ($DATA->{'nickcount'}{$pot_nick}))
			{
				if (!defined ($DATA->{'count'}{$pot_nick}))
				{
					$DATA->{'count'}{$pot_nick} =
					{
						usedby	=> $nick,
						count	=> 1
					}
				}
				else
				{
					$DATA->{'count'}{$pot_nick}{'usedby'} = $nick;
					$DATA->{'count'}{$pot_nick}{'count'}++;
				}
			}
		}
	}
	else { die; }

	return (1);
}

sub calculate
{
	my $retval = [];
	
	my $max = 10;
	if (get_config ('plugin_max'))
	{
		my $tmp = get_config ('plugin_max');
		$tmp =~ s/\D//g;

		$max = $tmp if ($tmp);
	}
	
	my @nicks = sort
	{
		$DATA->{'count'}{$b}{'count'} <=> $DATA->{'count'}{$a}{'count'}
	} (keys %{$DATA->{'count'}});

	foreach (@nicks)
	{
		next unless (defined ($DATA->{'nickcount'}{$_}));
		next unless ($DATA->{'nickcount'}{$_} > 100);
		
		my $usedby = $DATA->{'count'}{$_}{'usedby'};
		my $count  = $DATA->{'count'}{$_}{'count'};

		push (@$retval, [$_, $count, $usedby]);

		$max--;
		last unless ($max);
	}

	return ($retval);
}

sub output
{
	my $data = calculate ();

	return (undef) unless (@$data);
	
	my $fh = get_filehandle ();

	my $nick = translate ('Nick');
	my $times = translate ('Times used');
	my $last = translate ('Last used by');
	
	print $fh <<EOF;
<table class="plugin">
  <tr>
    <td class="invis">&nbsp;</td>
    <th>$nick</th>
    <th>$times</th>
    <th>$last</th>
  </tr>
EOF
	my $i = 0;
	foreach (@$data)
	{
		$i++;
		my ($nick, $count, $usedby) = @$_;

		print $fh "  <tr>\n",
		qq#    <td class="numeration">$i</td>\n#,
		qq#    <td>$nick</td>\n#,
		qq#    <td>$count</td>\n#,
		qq#    <td>$usedby</td>\n#,
		qq#  </tr>\n#;
	}

	print $fh "</table>\n\n";
}
